'use strict';

var fetchres = require('fetchres');
var logger = require('ft-next-logger');
var eagerFetch = require('./utils/eager-fetch');
var elasticAgent = require('./utils/elastic-agent');

module.exports = function(opts) {
	var query = opts.query;
	var fields = opts.fields;
	var count = opts.count || 10;
	var offset = opts.offset || 0;
	var url = 'http://api.ft.com/content/search/v1';
	var useElasticSearch = opts.useElasticSearch;
	var body;

	if (useElasticSearch) {
		var queryObject;
		if (typeof query === 'object') {
			queryObject = query;
		} else {
			var convertedQuery = /^([a-zA-Z]+):"(.*)"$/.exec(query) || [];
			var taxonomy = convertedQuery[1].replace(/Id$/, '');
			var filter = 'item.metadata.' + taxonomy + '.term.' + (/Id$/.test(convertedQuery[1]) ? 'id' : 'name');
			var id = convertedQuery[2];
			var term = {};
			term[filter] = id;
			queryObject = {
				filtered: {
					filter: {
						term: term
					}
				}
			};
		}
		body = {
			fields: fields || [],
			from: offset,
			size: count,
			query: queryObject,
			sort: [
				{
					'item.lifecycle.lastPublishDateTime': {
						order: "desc"
					}
				}
			]
		};
		return eagerFetch('https://' + process.env.ELASTIC_SEARCH_HOST + '/v1_api_v2/item/_search', {
				method: 'POST',
				body: JSON.stringify(body),
				timeout: 3000,
				agent: elasticAgent
			})
			.then(function(response) {
				if (!response.ok) {
					logger.warn('Failed getting SAPIv1 content from Elastic Search', {
						query: query,
						status: response.status
					});

				}
				return response;
			})
			.then(fetchres.json)
			.then(function(results) {
				var indexCount = results.hits.total;
				results = results.hits.hits.map(function(hit) {
					return fields ? hit : hit._id;
				});
				results.indexCount = indexCount;
				return results;
			});
	}

	if (typeof query === 'object' || fields) {
		return Promise.reject('Complex searches are only supported when elastic search is turned on');
	}
	body = {
		queryString: query,
		queryContext: {
			curations: ["ARTICLES", "BLOGS"]
		},
		resultContext: {
			maxResults: count,
			offset: offset,
			sortOrder: "DESC",
			sortField: "lastPublishDateTime"
		}
	};

	return eagerFetch(url, {
			timeout: 3000,
			body: JSON.stringify(body),
			method: 'post',
			headers: {
				'Content-Type': 'application/json',
				'X-Api-Key': process.env.apikey
			}
		})
			.then(function(response) {
				if (!response.ok) {
					logger.warn('Failed getting SAPIv1 content', {
						query: query,
						status: response.status
					});

				}
				return response;
			})
			.then(fetchres.json)
			.then(function(result) {
				var indexCount = result.results[0].indexCount;
				if (indexCount === 0) {
					var output = [];
					output.indexCount = 0;
					return output;
				}
				return Promise.all(result.results[0].results.map(function(article) {
						return article.id;
					}))
					.then(function(articles) {
						articles = articles.filter(function(article) {
							return article;
						});
						articles.indexCount = indexCount;
						return articles;
					});
			});
};
